package lcsb.mapviewer.api.controller;

import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import lcsb.mapviewer.api.BaseController;
import lcsb.mapviewer.common.Configuration;
import lcsb.mapviewer.services.SecurityException;
import lcsb.mapviewer.services.interfaces.IUserService;
import lcsb.mapviewer.services.view.AuthenticationToken;

@RestController
@RequestMapping("/user")
public class UserController extends BaseController {
	Logger							 logger	= Logger.getLogger(UserController.class);

	@Autowired
	private IUserService userService;

	@Autowired
	private UserRestImpl userRest;

	@RequestMapping(value = "/login", method = { RequestMethod.GET, RequestMethod.POST }, produces = { MediaType.APPLICATION_JSON_VALUE })
	public AuthenticationToken login(@RequestParam(value = "login", defaultValue = Configuration.ANONYMOUS_LOGIN) String login,
			@RequestParam(value = "password", required = false) String password) {
		AuthenticationToken token = userService.login(login, password);
		return token;
	}

	@RequestMapping(value = "/tokenStatus", method = { RequestMethod.GET, RequestMethod.POST }, produces = { MediaType.APPLICATION_JSON_VALUE })
	public AuthenticationToken tokenSatus(@RequestParam(value = "token", required = false) String token) throws SecurityException {
		return userService.getToken(token);
	}

	@RequestMapping(value = "/getUser", method = { RequestMethod.GET, RequestMethod.POST }, produces = { MediaType.APPLICATION_JSON_VALUE })
	public Map<String, Object> getUser(@RequestParam(value = "token", required = false) String token,
			@RequestParam(value = "userId", defaultValue = "") String userId,
			@RequestParam(value = "columns", defaultValue = "") String columns) throws SecurityException {
		return userRest.getUser(token, userId, columns);
	}

	@RequestMapping(value = "/logout", method = { RequestMethod.GET, RequestMethod.POST }, produces = { MediaType.APPLICATION_JSON_VALUE })
	public Map<String, String> logout(@RequestParam(value = "token", required = false) String token) throws SecurityException {
		userService.logout(token);
		Map<String, String> response = new HashMap<>();
		response.put("status", "OK");
		return response;
	}

	/**
	 * @return the userService
	 * @see #userService
	 */
	public IUserService getUserService() {
		return userService;
	}

	/**
	 * @param userService
	 *          the userService to set
	 * @see #userService
	 */
	public void setUserService(IUserService userService) {
		this.userService = userService;
	}

	/**
	 * @return the userRest
	 * @see #userRest
	 */
	public UserRestImpl getUserRest() {
		return userRest;
	}

	/**
	 * @param userRest
	 *          the userRest to set
	 * @see #userRest
	 */
	public void setUserRest(UserRestImpl userRest) {
		this.userRest = userRest;
	}
}