package lcsb.mapviewer.api.project;

import java.awt.geom.Point2D;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import lcsb.mapviewer.api.BaseController;
import lcsb.mapviewer.api.QueryException;
import lcsb.mapviewer.services.SecurityException;

@RestController
@RequestMapping("/project")
public class ProjectController extends BaseController {
	@Autowired
	private ProjectRestImpl projectController;

	@RequestMapping(value = "/getMetaData", method = { RequestMethod.GET, RequestMethod.POST }, produces = { MediaType.APPLICATION_JSON_VALUE })
	public ProjectMetaData getMetaData(@RequestParam(value = "projectId") String projectId, @RequestParam(value = "token") String token)
			throws SecurityException {
		return projectController.getMetaData(projectId, token);
	}

	@RequestMapping(value = "/getSuggestedQueryList", method = { RequestMethod.GET, RequestMethod.POST }, produces = { MediaType.APPLICATION_JSON_VALUE })
	public String[] getSuggestedQueryList(@RequestParam(value = "projectId") String projectId, @RequestParam(value = "token") String token)
			throws SecurityException {
		return projectController.getSuggestedQueryList(projectId, token);
	}

	@RequestMapping(value = "/getElements", method = { RequestMethod.GET, RequestMethod.POST }, produces = { MediaType.APPLICATION_JSON_VALUE })
	public List<Map<String, Object>> getElements(@RequestParam(value = "projectId") String projectId, @RequestParam(value = "id", defaultValue = "") String id,
			@RequestParam(value = "columns", defaultValue = "") String columns, @RequestParam(value = "token") String token) throws SecurityException {
		return projectController.getElements(projectId, id, columns, token);
	}

	@RequestMapping(value = "/getReactions", method = { RequestMethod.GET, RequestMethod.POST }, produces = { MediaType.APPLICATION_JSON_VALUE })
	public List<Map<String, Object>> getReactions(@RequestParam(value = "projectId") String projectId, @RequestParam(value = "id", defaultValue = "") String id,
			@RequestParam(value = "columns", defaultValue = "") String columns, @RequestParam(value = "token") String token) throws SecurityException {
		return projectController.getReactions(projectId, id, columns, token);
	}

	@RequestMapping(value = "/getClosestElementsByCoordinates", method = { RequestMethod.GET, RequestMethod.POST },
			produces = { MediaType.APPLICATION_JSON_VALUE })
	public List<Map<String, Object>> getClosestElementsByCoordinates(@RequestParam(value = "projectId") String projectId,
			@RequestParam(value = "modelId") String modelId, @RequestParam(value = "token") String token, @RequestParam(value = "coordinates") String coordinates,
			@RequestParam(value = "count", defaultValue = "5") String count) throws QueryException, SecurityException {
		String[] tmp = coordinates.split(",");
		if (tmp.length != 2) {
			throw new QueryException("Coordinates must be in the format: 'xxx.xx,yyy.yy'");
		}
		Double x = null;
		Double y = null;
		try {
			x = Double.valueOf(tmp[0]);
			y = Double.valueOf(tmp[1]);
		} catch (NumberFormatException e) {
			throw new QueryException("Coordinates must be in the format: 'xxx.xx,yyy.yy'", e);
		}
		Point2D pointCoordinates = new Point2D.Double(x, y);
		return projectController.getClosestElementsByCoordinates(projectId, modelId, token, pointCoordinates, Integer.valueOf(count));
	}

	@RequestMapping(value = "/getElementsByQuery", method = { RequestMethod.GET, RequestMethod.POST }, produces = { MediaType.APPLICATION_JSON_VALUE })
	public List<Map<String, Object>> getElementsByQuery(@RequestParam(value = "projectId") String projectId, @RequestParam(value = "token") String token,
			@RequestParam(value = "query") String query, @RequestParam(value = "maxElements", defaultValue = "100") Integer maxElements,
			@RequestParam(value = "perfectMatch", defaultValue = "false") String perfectMatch) throws QueryException, SecurityException {
		return projectController.getElementsByQuery(projectId, token, query, maxElements, perfectMatch);
	}

}