package lcsb.mapviewer.api.project;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyString;

import java.awt.geom.Point2D;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;

import lcsb.mapviewer.api.RestTestFunctions;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.services.interfaces.IModelService;

public class ProjectRestImplTest extends RestTestFunctions {
	Logger					logger = Logger.getLogger(ProjectRestImplTest.class);

	@Autowired
	ProjectRestImpl	_projectRestImpl;

	@Autowired
	IModelService		modelService;

	@AfterClass
	public static void tearDownAfterClass() throws Exception {
	}

	@Before
	public void setUp() throws Exception {
	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void testGetElementsProcessAllColumns() throws Exception {
		try {
			ProjectRestImpl projectRest = createMockProjectRest("testFiles/model/sample.xml");
			for (Map<String, Object> element : projectRest.getElements("sample", "", "", token.getId())) {
				for (String paramName : element.keySet()) {
					Object val = element.get(paramName);
					String paramValue = "";
					if (val != null) {
						paramValue = val.toString();
					}
					assertFalse("Improper handler for column name: " + paramName, paramValue.contains("Unknown column"));
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testGetClosestElementsByCoordinates() throws Exception {
		try {
			ProjectRestImpl projectRest = createMockProjectRest("testFiles/model/sample.xml");
			int count = 3;
			List<Map<String, Object>> result = projectRest.getClosestElementsByCoordinates("sample", "0", token.getId(), new Point2D.Double(1, 2), count);
			assertEquals(count, result.size());
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	private ProjectRestImpl createMockProjectRest(String string) throws Exception {
		Model model = super.getModelForFile(string, true);
		IModelService mockModelService = Mockito.mock(IModelService.class);
		Mockito.when(mockModelService.getLastModelByProjectId(anyString(), any())).thenReturn(model);
		_projectRestImpl.setModelService(mockModelService);
		return _projectRestImpl;
	}

}
